package activity

import (
	"gorm.io/gorm"
	"hongze/hongze_yb/models/tables/yb_activity"
	"hongze/hongze_yb/models/tables/yb_activity_permission"
	"hongze/hongze_yb/models/tables/yb_activity_register"
	"hongze/hongze_yb/models/tables/yb_activity_remind"
	"hongze/hongze_yb/models/tables/yb_activity_voice"
	"hongze/hongze_yb/services/company"
	"hongze/hongze_yb/services/user"
	"hongze/hongze_yb/utils"
	"log"
	"time"
)

var WeekDay = [7]string{"周日", "周一", "周二", "周三", "周四", "周五", "周六"}

// PageList 活动分页列表
func PageList(condition string, pars []interface{}, page, limit int, order string, userInfo user.UserInfo) (list []yb_activity.ActivityItem, err error) {
	activities, err := yb_activity.GetPageListByWhere(condition, pars, page, limit, order, int(userInfo.UserID))
	if err != nil {
		log.Print(err.Error())
		return
	}
	if activities != nil {
		var activityIds []int
		for _, item := range activities {
			activityIds = append(activityIds, item.ActivityID)
		}

		// 录音
		voices, queryErr := yb_activity_voice.GetVoicesByActivityIds(activityIds)
		if queryErr != nil {
			log.Print(queryErr.Error())
			return
		}
		voiceMap := make(map[int]int, 0)
		for _, voice := range voices{
			voiceMap[int(voice.ActivityID)] = 1
		}
		// 提醒
		reminds, queryErr := yb_activity_remind.GetRemindsByUserIdAndActivityIds(int(userInfo.UserID), activityIds)
		if queryErr != nil {
			log.Print(queryErr.Error())
			return
		}
		remindMap := make(map[int]int, 0)
		for _, remind := range reminds{
			remindMap[remind.ActivityId] = 1
		}
		// 报名
		registers, queryErr := yb_activity_register.GetRegistersByUserIdAndActivityIds(int(userInfo.UserID), activityIds)
		if queryErr != nil {
			log.Print(queryErr.Error())
			return
		}
		registerMap := make(map[int]int, 0)
		for _, register := range registers{
			if register.RegisterState == 1 {
				registerMap[register.ActivityId] = 1
			}
		}

		var temp yb_activity.ActivityItem
		for _, activity := range activities {
			temp = activity
			temp.WeekString = WeekDay[activity.StartTime.Weekday()]
			temp.ActivityState = getActivityStateByTime(activity.StartTime, activity.EndTime)
			// 是否有回放
			tempActivity := activity
			_, ok := voiceMap[tempActivity.ActivityID]
			if ok {
				temp.HasPlayBack = 1
			}
			// 是否有提醒
			_, ok = remindMap[tempActivity.ActivityID]
			if ok {
				temp.HasRemind = 1
			}
			// 是否已报名
			_, ok = registerMap[tempActivity.ActivityID]
			if ok {
				temp.RegisterState = 1
			}
			list = append(list, temp)
		}
	}
	return
}

// getActivityStateByTime 根据时间生成活动枚举值
func getActivityStateByTime(startTime, endTime time.Time) (state int) {
	weekStart := utils.GetNowWeekMonday()
	weekEnd := utils.GetNowWeekLastDay()
	nowTime := time.Now().Format("2006-01-02 15:04:05")
	timeNow, _ := time.ParseInLocation("2006-01-02 15:04:05", nowTime, time.Local)
	if timeNow.Before(endTime) && timeNow.After(startTime) {
		// 进行中
		state = 2
	} else if timeNow.After(endTime) {
		// 已结束
		state = 3
	} else if timeNow.Before(weekEnd) && timeNow.Before(endTime) && timeNow.After(weekStart) && timeNow.Before(startTime) {
		// 本周预告
		state = 1
	}
	return
}

// GetActivityDetail 活动详情
func GetActivityDetail(activityId, userId int) (detail *yb_activity.ActivityDetail, err error) {
	detail, err = yb_activity.GetDetailById(activityId)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return
		}
	}
	if detail == nil {
		return
	}
	registeredNum, err := yb_activity_register.CountRegistedNumByActivityId(activityId)
	if err != nil {
		log.Print(err.Error())
		return
	}
	detail.RegisteredNum = int(registeredNum)
	detail.WeekString = WeekDay[detail.StartTime.Weekday()]
	detail.ActivityState = getActivityStateByTime(detail.StartTime, detail.EndTime)
	_, remindErr := yb_activity_remind.RemindExist(userId, activityId)
	if remindErr != gorm.ErrRecordNotFound {
		detail.HasRemind = 1
	}
	_, registerErr := yb_activity_register.RegisterExist(userId, activityId)
	if registerErr != gorm.ErrRecordNotFound {
		detail.RegisterState = 1
	}
	return
}

// CheckActivityPermission 验证用户的活动参与权限
func CheckActivityPermission(userInfo user.UserInfo, activityId int) (ok bool, permissionCheckInfo company.PermissionCheckInfo, err error) {
	companyId := userInfo.CompanyID
	activity, err := yb_activity.GetOneById(activityId)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return
		}
	}
	if activity == nil {
		return
	}
	// 查询活动权限集合
	permissions, err := yb_activity_permission.GetPermissionsByActivityId(activityId)
	if err != nil {
		return
	}
	var permissionIds []int
	for _, item := range permissions {
		pid := int(item.PermissionID)
		permissionIds = append(permissionIds, pid)
	}
	ok, permissionCheckInfo, err = company.CheckPermissionByPermissionIdList2Ficc(companyId, permissionIds)
	return
}